#!/usr/bin/perl 

# 
#  FASt Term Recognizer
# 
#   DISTRIB/bin/TermtoRules.pl
#   Version 2.04/04-02-01
# 
#   Copyright (C) 2004  Christian Jacquemin, LIMSI-CNRS
# 
#   Transformation of candidate terms into rules for FASTR
# 

# STDIN: candidate terms
#
# 00000[\t]domain document [\t]NN NNS 
# 00001[\t]entire sentence [\t]JJ NNS 
# 00002[\t]high information [\t]JJ NN 

# IN: FILETags = file for tag conversion
#
# #
#    <cat> = Pu.\n
# # punctuation
# $
#    <cat> = Pu.\n
# # punctuation
# ''
#    <cat> = Pu.\n
# # punctuation

# STDOUT: lemma rules + term rules into FASTR format
# 
# Word 'domain'                         : <cat> =   N.
# Word 'document'                       : <cat> =   N.
# Rule N1 -> N2 N3:
#  <N1 lex> = 'N2'
#  <N1 lab> = '00000'
#  <N2 lem> = 'domain'
#  <N3 lem> = 'document'.

# syntactic check
if( (@ARGV + 0) != 1 ) {
    die "Usage: TreeTaggertoFastr.pl <tag-file>!\n";
}
 
# reading the extended tags
$FILENAMETAGS = shift @ARGV;
open( FILETags , "<$FILENAMETAGS" ) 
    ||  die "Can't open file ($FILENAMETAGS)!" ;

%TAGEQUIVALENCE = ();
while( <FILETags> ){
    # TreeTagger tag
    chop;
    $TAGORIG = $_;
    if( !($_ = <FILETags>) ){
	break;
    }

    # fastr tag / features
    chop;
    $NEWTAG = $_;

    # comments
    if( !<FILETags> ){
	break;
    }

    # only keeps the cateogory and discard the features
    # stores the translation
    $NEWTAG =~ s/.*<cat> *= *([A-Za-z]+).*/$1/;
    $TAGEQUIVALENCE{ $TAGORIG } = $NEWTAG;
}
close( FILETags );

sub PrintRules {
}

while(<>) {
    chop;
    ( $LABEL , $LEMMAS , $CATEGORIES ) = split( /\t/ );
    @TAGGER_CATEGORIES = split( / / , $CATEGORIES );
    @LEMMAS = split( / / , $LEMMAS );

    @CATEGORIES = ();
    foreach $TAG ( @TAGGER_CATEGORIES ) {
	$TAG = $TAGEQUIVALENCE{ $TAG };
	push( @CATEGORIES ,  $TAG );
    }

    # the first substantive == the anchor for lexicalisation
    $ROWFIRSTNOUN = -1;
    $ROWFIRSTSUB = -1;
    $ROWFIRSTX = -1;

    # initialization of the loop 
    $COUNT = 0;
    # loop for collecting the words and searching the anchor of lexicalization
    foreach $CAT ( @CATEGORIES ) {
	if( ( $ROWFIRSTNOUN == -1 ) && ( $CAT eq "N" ) ) {
	    $ROWFIRSTNOUN = $COUNT;
	}
	if( ( $ROWFIRSTSUB == -1 ) 
	    && ( ( $CAT eq "V" ) || ( $CAT eq "Av" )
		 || ( $CAT eq "A" ) ) ) {
	    $ROWFIRSTSUB = $COUNT;
	}
	if( ( $ROWFIRSTX == -1 ) && ( $CAT eq "X" ) ) {
	    $ROWFIRSTX = $COUNT;
	}
	$COUNT++;
    }
    # emergency lexicalisation
    if( ($ROWFIRSTNOUN == -1) && ($ROWFIRSTSUB == -1) ) {
	$ROWFIRSTNOUN = $ROWFIRSTX;   
	if( $ROWFIRSTNOUN == -1 ) {
	    $ROWFIRSTNOUN = 0;
	}
    }
    else {
	if( $ROWFIRSTNOUN == -1 ) {
	    $ROWFIRSTNOUN = $ROWFIRSTSUB;
	}   
    }

    # printing the lemma rules
    $COUNT = 0;
    foreach $LEMMA ( @LEMMAS ) {
	$STEM = $LEMMA;
	$STEM =~ s/\\/\\\\/g;
	$STEM =~ s/\'/\\\'/g;
	printf 
	    "Word '%s' %s: <cat> = %3s.\n"
		, $STEM , " " x ( 30 - length( $STEM ) ) 
		    , $CATEGORIES[ $COUNT ];
	$COUNT++;
    }

    # printing the term rule
    printf "Rule N1 ->";
    $COUNT = 2;
    foreach $CAT ( @CATEGORIES ) {
	printf " %s%d" , $CAT , $COUNT;
	$COUNT++;
    }
    printf ":\n";
    
    printf " <N1 lex> = '%s%d'\n" , 
	$CATEGORIES[ $ROWFIRSTNOUN ] , $ROWFIRSTNOUN + 2;
    printf " <N1 lab> = '%s'" , $LABEL;
   
    # loop for creating the constraints of the term rule
    $COUNT = 0;
    foreach $LEMMA ( @LEMMAS ) {
	$STEM = $LEMMA;
	$STEM =~ s/\\/\\\\/g;
	$STEM =~ s/\'/\\\'/g;
	printf "\n <%s%d lem> = '%s'" , 
	     $CATEGORIES[ $COUNT ] , $COUNT + 2 , $STEM;
        $COUNT++;
    }
    printf ".\n";
}
